import { CollabFail, CollabMessageBus } from '@bangle.dev/collab-comms';
import { Node, Step, Schema } from '@bangle.dev/pm';
import { EitherType } from '@bangle.dev/utils';

interface StepBigger extends Step {
    clientID: string;
}
declare const MAX_STEP_HISTORY = 1000;
declare class CollabServerState {
    readonly doc: Node;
    readonly steps: StepBigger[];
    readonly version: number;
    static addEvents(state: CollabServerState, version: number, steps: Step[], clientID: string): EitherType<CollabFail, CollabServerState>;
    static getEvents(collabState: CollabServerState, version: number): EitherType<CollabFail, {
        version: number;
        steps: StepBigger[];
    }>;
    static isInvalidVersion(collabState: CollabServerState, version: number): boolean;
    constructor(doc: Node, steps?: StepBigger[], version?: number);
}

declare type ApplyState = (docName: string, newCollabState: CollabServerState, oldCollabState: CollabServerState) => boolean;
declare class CollabManager {
    private _options;
    readonly managerId: string;
    private readonly _abortController;
    private _handleRequest;
    private readonly _instances;
    private readonly _serverCom?;
    constructor(_options: {
        applyState?: ApplyState;
        getInitialState: (docName: string) => Promise<CollabServerState | undefined>;
        collabMessageBus: CollabMessageBus;
        managerId?: string;
        schema: Schema;
    });
    destroy(): void;
    getAllDocNames(): Set<string>;
    getCollabState(docName: string): CollabServerState | undefined;
    isDestroyed(): boolean;
    resetDoc(docName: string): void;
    private _createInstance;
    /**
     * Get an instance of a document or creates a new one
     * if none exists.
     * @param docName
     * @returns
     */
    private _getInstance;
    private _handleGetDocument;
    private _handlePullEvents;
    private _handlePushEvents;
    private _toResponse;
}

export { CollabManager, CollabServerState, MAX_STEP_HISTORY };
